import { AsyncServiceImplementationError } from "./errors/AsyncServiceImplementationError.js";
import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { Debug } from "../debug.js";
const NOT_LOADED = Symbol("NOT_LOADED");
export class AsyncServiceImplementationLoader {
    loader;
    get name() {
        if (this.implementation === NOT_LOADED) {
            return Debug.errors.serviceNotLoaded(this.loader);
        }
        return getNameOfServiceType(this.implementation);
    }
    implementation;
    promise;
    constructor(loader) {
        this.loader = loader;
        this.implementation = NOT_LOADED;
        this.promise = null;
    }
    isLoaded() {
        return this.implementation !== NOT_LOADED;
    }
    getImplementation() {
        if (this.implementation === NOT_LOADED) {
            throw new AsyncServiceImplementationError();
        }
        return this.implementation;
    }
    async load() {
        if (this.implementation !== NOT_LOADED) {
            return this.implementation;
        }
        if (!this.promise) {
            this.promise = this.loader()
                .then((implementation) => {
                this.implementation = implementation;
                return implementation;
            })
                .finally(() => {
                this.promise = null;
            });
        }
        return this.promise;
    }
}
//# sourceMappingURL=AsyncServiceImplementationLoader.js.map