import { IServiceProvider } from "./IServiceProvider.js";
import { ServiceLifetime } from "./ServiceLifetime.js";
import { tryResolveService } from "./validation/tryResolveService.js";
import { Debug } from "../debug.js";
export class ServiceCollection {
    collection;
    constructor(collection) {
        this.collection = new Map(collection?.copy() || []);
        if (!this.collection.has(IServiceProvider)) {
            this.addSingleton(IServiceProvider, function IServiceProvider() {
                throw new Error("Not implemented");
            });
        }
    }
    *[Symbol.iterator]() {
        for (const descriptors of this.collection.values()) {
            for (const descriptor of descriptors) {
                yield descriptor;
            }
        }
    }
    getDescriptors(service) {
        return (this.collection.get(service) ?? []);
    }
    addScoped(service, implementation) {
        if (implementation === undefined) {
            implementation = service;
        }
        this.addService(ServiceLifetime.Scoped, service, implementation);
        return this;
    }
    addTransient(service, implementation = service) {
        this.addService(ServiceLifetime.Transient, service, implementation);
        return this;
    }
    addSingleton(service, implementation = service) {
        this.addService(ServiceLifetime.Singleton, service, implementation);
        return this;
    }
    *copy() {
        for (const [key, descriptors] of this.collection) {
            yield [key, [...descriptors]];
        }
    }
    addService(lifetime, service, implementation) {
        const descriptors = this.collection.get(service) ?? [];
        this.collection.set(service, [
            ...descriptors,
            {
                service,
                lifetime,
                implementation,
                loader: null,
            },
        ]);
        if (Debug.extended) {
            tryResolveService(this, service, new Set());
        }
        return this;
    }
}
//# sourceMappingURL=ServiceCollection.js.map